include <net-snmp/net-snmp-config.h>
#include <net-snmp/net-snmp-includes.h>
#include <string.h>

/*
 * siehe auch: http://www.net-snmp.org/tutorial/tutorial-5/toolkit/index.html
 */

int main(int argc, char ** argv)
{
	struct snmp_session session, *ss;
	struct snmp_pdu *pdu;
	struct snmp_pdu *response;

	oid anOID[MAX_OID_LEN];
	size_t anOID_len = MAX_OID_LEN;

	struct variable_list *vars;
	int status;
	int count=1;

	/* Initialisierung der SNMP Bibliothek */
	init_snmp("snmpapp");

	/* Initialisierung der Session */
	snmp_sess_init( &session );

	/*
	 * Nach Initialisierung setzt man die Paramter der Session.
	 * Dazu gehren Hostname, Version, sowie Community String, der als
	 * Passwort benutzt wird.
	 */
	session.peername = strdup("localhost");
	session.version = SNMP_VERSION_1;
	session.community = "public"; /* Authentifizierung */
	session.community_len = strlen(session.community);

	/*
	 * Anschlieend die Sitzung ffnen.
	 */
	SOCK_STARTUP; 
	ss = snmp_open(&session);

	if (!ss) {
		snmp_perror("ack");
		snmp_log(LOG_ERR, "Verbindung konnte nicht aufgebaut werden!\n");
		exit(2);
	}

	/*
 	 * Fr die Datenabfrage ist zunchst ein Datenpaket mit dem gewnschten
 	 * Nachrichtentyp zu erstellen. Danach wird die OID mit einer 
 	 * Funktion auf ein Standardformat gebracht und in das Paket 
	 * geschrieben.
	 * Wir wollen die Beschreibung des Systems:
	 * 	system.sysDescr.0
	 */
	 
	pdu = snmp_pdu_create(SNMP_MSG_GET);
	read_objid(".1.3.6.1.2.1.1.1.0", anOID, &anOID_len);

	/* 
	 * andere Mglicheiten sind:
	 *	get_node("sysDescr.0", anOID, &anOID_len);
	 *	read_objid("system.sysDescr.0", anOID, &anOID_len);
	 */

	snmp_add_null_var(pdu, anOID, anOID_len);

	/*
	 * Die Anfrage ist jetzt vollstndig und kann gesendet werden.
	 */
	status = snmp_synch_response(ss, pdu, &response);

	/*
	 * Verarbeiten der Antwort vom Server
	 */
	if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
		/*
		 * Erfolg. Anzeigen der Variablen
		 */

		for(vars = response->variables; vars; vars = vars->next_variable)
			print_variable(vars->name, vars->name_length, vars);
		/*
		 * eigene Ausgabe von String Daten
		 */
		for(vars = response->variables; vars; vars = vars->next_variable) {
			if (vars->type == ASN_OCTET_STR) {
				char *sp = (char *)malloc(1 + vars->val_len);
				memcpy(sp, vars->val.string, vars->val_len);
				sp[vars->val_len] = '\0';
				printf("Wert Nr. %d ist ein String: %s\n", count++, sp);
				free(sp);
			}
			else
				printf("Wert Nr. %d ist kein String!\n", count++);
		}
	} else {
		/*
		 * Fehler. Anzeigen des Fehlers
		 */

		if (status == STAT_SUCCESS)
			fprintf(stderr, "Fehler!\nGrund: %s\n",
					snmp_errstring(response->errstat));
		else
			snmp_sess_perror("snmpget", ss);

	}

	/*
	 * Aufraeumen:
	 * 1) Freigeben der Datenstruktur 'response'
	 * 2) Schlieen der Session.
	 */
	if (response)
		snmp_free_pdu(response);
	snmp_close(ss);

	SOCK_CLEANUP;
	return (0);
} /* main() */
